io.stdout:setvbuf("no")

local lg = love.graphics
lg.setDefaultFilter("nearest")

runOnce = true
local g3d = require "g3d"
local Player = require "player"
local vectors = require "g3d/vectors"
local primitives = require "primitives"

local map, background, player
local canvas
local accumulator = 0
local frametime = 1/60
local rollingAverage = {}


grid = {}
runOnce = true


function love.load()
    min_dt = 1/60
    next_time = love.timer.getTime()

    blocks = {{"Grass Block", "assets/grass_block.png"}, {"Stone Block", "assets/stone_block.png"}}
    blockList = {}
    lg.setBackgroundColor(0.25,0.5,1)

    renderBlockList = {}

    --map = g3d.newModel("assets/map.obj", "assets/tileset.png", {0, -3, 0}, nil, {-1,-1,1})
    background = g3d.newModel("assets/sphere.obj", "assets/starfield.png", {0,0,0}, nil, {500,500,500})
    player = Player:new(0,0,0)
    --player:addCollisionModel(map)
    player:addCollisionModel(block)
    
    canvas = {lg.newCanvas(love.graphics.getWidth(),love.graphics.getHeight()), depth=true}
    noise()
    for i = 1, 5 do
        for x = 1, #grid do
            for y = 1, #grid[x] do
                local f = 2 * grid[x][y] - 1
                local _x = x - 1
                table.insert(blockList, {1, {_x,math.ceil(f),y - 1}})
                table.insert(blockList, {2, {_x,math.ceil(f - i),y - 1}})
            end
        end
    end
    for i=1,#blockList do
        
        block = g3d.newModel("assets/cube.obj", blocks[blockList[i][1]][2], {0.4 * blockList[i][2][1],  0.71 + -0.4 * blockList[i][2][2], 0.4 * blockList[i][2][3]}, nil, {0.20, 0.20, 0.20})
        player:addCollisionModel(block)
        table.insert(renderBlockList, block)
    end
    collectgarbage()
end

function addToBlockMesh(blockID, position)

    player:addCollisionModel(block)
end

function deleteFromBlockMesh(position)

    player:addCollisionModel(block)
end

function noise()
    for x = 1, 16 do
        for y = 1, 16 do
            grid[x] = grid[x] or {}
            grid[x][y] = love.math.noise(x + love.math.random(), y + love.math.random())
        end
    end
end

function love.update(dt)
    next_time = next_time + min_dt

    table.insert(rollingAverage, dt)
    if #rollingAverage > 60 then
        table.remove(rollingAverage, 1)
    end
    local avg = 0
    for i,v in ipairs(rollingAverage) do
        avg = avg + v
    end

    -- fixed timestep accumulator
    accumulator = accumulator + avg/#rollingAverage
    while accumulator > frametime do
        accumulator = accumulator - frametime
        player:update(dt)
    end

    -- interpolate player between frames
    player:interpolate(accumulator/frametime)
    background:setTranslation(g3d.camera.position[1],g3d.camera.position[2],g3d.camera.position[3])

    pX = player:getXYZ()[1]
    pY = player:getXYZ()[2]
    pZ = player:getXYZ()[3]
end

function love.keypressed(k)
    if k == "escape" then love.event.push("quit") end
    keyPressed = k
    noise()
end

function love.mousemoved(x,y, dx,dy)
    g3d.camera.firstPersonLook(dx,dy)
end

local function setColor(r,g,b,a)
    lg.setColor(r/255, g/255, b/255, a and a/255)
end

function love.draw()
    lg.setCanvas(canvas)
    lg.clear(0,0,0,0)

    --lg.setDepthMode("lequal", true)
    for i=1,#renderBlockList do
        if renderBlockList[i]:sphereIntersection(pX, pY, pZ, 1) then
        renderBlockList[i]:draw()
        end


    end
    background:draw()

    --map:draw()

    lg.setColor(1,1,1)

    lg.setCanvas()
    lg.draw(canvas[1], love.graphics.getWidth()/2, love.graphics.getHeight()/2, 0, 1,-1, love.graphics.getWidth()/2, love.graphics.getHeight()/2)
    lg.print("GC: " .. collectgarbage("count"), 0, 80)
    lg.print("FPS: " .. tostring(love.timer.getFPS()))
    lg.print("ESC to exit", 0, 25)
    lg.print(tostring(math.floor(pX * 2)), 0, 60)
    lg.print(tostring(math.floor(pY)), 15, 60)
    lg.print(tostring(math.floor(pZ * 2)), 35, 60)

    local cur_time = love.timer.getTime()
    if next_time <= cur_time then
       next_time = cur_time
       return
    end
    love.timer.sleep(next_time - cur_time)
end
